%{

//-------------------------------------------------------------------
// TDL.l
//
// This file is a standard input file to flex (which is a 
// variant of lex).  It contains token definitions for the 
// IEEE 1451.4 Template Description Language (TDL). This file 
// should be used in conjunction with bison and its TDL input 
// file tdl.y.  To use this file execute the command line:
//
//		flex -i tdl.l 
//
// or the command line 
//
//		os2 /p flex.exe /c flex -i tdl.l
//
// After executing this, then 
//
//		rename lexyy.c lexyy.cpp
//
// Then execute bison on the tdl.y file and then compile the main
// TDL compiler using a C++ compiler.
//
// Note: It has been reported that some versions of flex generate
// code which includes a #include <unistd.h> statement.  If such a
// file does not exist, then create a blank file of this name in
// the working directory.
//
// Updated 10 September 2003
//
//-------------------------------------------------------------------

#include "TDL_tab.h"
#include "tdlparser.h"

extern int statement_count;
extern void print_statement_num ();
extern void print_token (char*);

%}

%%

TDL_VERSION_NUMBER  { print_token ("TDL VERSION NUMBER"); 
                      return(TDL_VERSION_NUMBER);}
VALIDATION_KEYCODE  { print_token ("VALIDATION_KEYCODE"); 
                      return(VALIDATION_KEYCODE);}
UGID            { print_token ("UGID"); return(UGID);}
SPACING         { print_token ("SPACING"); return(SPACING);}
SELECTCASE      { print_token ("SELECTCASE"); return(SELECTCASE);}
ENDSELECT       { print_token ("ENDSELECT"); return(ENDSELECT);}
CASE            { print_token ("CASE"); return(CASE);}
ENDCASE	        { print_token ("ENDCASE"); return(ENDCASE);}
STRUCTARRAY     { print_token ("STRUCTARRAY"); return(STRUCTARRAY);}
ENDSTRUCTARRAY  { print_token ("ENDSTRUCTARRAY"); 
								return(ENDSTRUCTARRAY);}
ENUMERATE       { print_token ("ENUMERATE"); return(ENUMERATE);}
PHYSICAL_UNIT   { print_token ("PHYSICAL_UNIT"); 
								return(PHYSICAL_UNIT);}

ID           { print_token ("ID"); return(ID);}
CAL          { print_token ("CAL"); return(CAL);}
USR          { print_token ("USR"); return(USR);}
ALIGN        { print_token ("ALIGN"); return(ALIGN);}

UNINT        { print_token ("UNINT"); return(UNINT);}
CHR5         { print_token ("CHR5"); return(CHR5);}
STRING5      { print_token ("STRING5"); return(STRING5);}
STRING7      { print_token ("STRING7"); return(STRING7);}
STRING16     { print_token ("STRING16"); return(STRING16);}
DATE         { print_token ("DATE"); return(DATE);}
ConRelRes    { print_token ("ConRelRes"); return(CONRELRES);}
ConRes       { print_token ("ConRes"); return(CONRES);}
ASCII        { print_token ("ASCII"); return(ASCII);}
Single       { print_token ("Single"); return(SINGLE);}
Unicode      { print_token ("Unicode"); return(UNICOD);} // UNICODE conflicts with a C++ variable
BITBIN       { print_token ("BITBIN"); return(BITBIN);}

DEFAULT      { print_token ("DEFAULT	"); return(DEFAULT	);}
INITIALIZE   { print_token ("INITIALIZE"); return(INITIALIZE);}
READWRITE    { print_token ("READWRITE"); return(READWRITE);}
CTRLFUNCTIONMASK { print_token ("CTRLFUNCTIONMASK"); return(CTRLFUNCTIONMASK);}
FUNCTIONTYPE { print_token ("FUNCTIONTYPE"); return(FUNCTIONTYPE);}
FUNCTION     { print_token ("FUNCTION"); return(FUNCTION);}

ENDTEMPLATE     { print_token ("ENDTEMPLATE"); return(ENDTEMPLATE);}
ABSTRACT[^\n]*  { yylval.str=strdup(yytext); print_token (yylval.str);
                  return(ABSTRACT);}
TEMPLATE        { yylval.str=strdup(yytext); print_token (yylval.str);
                  return(TEMPLATE);}

\%[A-Za-z][A-Za-z0-9\^$_!@#&]*  { yylval.str=strdup(yytext); 
                                  print_token (yylval.str);
                                  return(PROPERTYNAME);}
\/\/[^\n]* { yylval.str=strdup(yytext); print_token (yylval.str);
             return(COMMENT);}
[A-Za-z][A-Za-z0-9\^$_!@#&]*  { yylval.str=strdup(yytext); 
                                print_token (yylval.str); 
                                return(VARIABLE_NAME);}

[\n]    {return (END_OF_LINE); }
[\t ]*  {}

\"[^"\n]*\"	|
\'[^'\n]*\'	{ yylval.str=strdup(yytext); print_token (yylval.str);
              return(QUOTED_STRING);}

\[    |
\]    |
\=    |
\,    |
\(    |
\)    { print_token (&yytext[0]); return(yytext[0]);}

[-+]?[0-9]+	       { yylval.str=strdup(yytext); print_token (yylval.str);
                     return(INTEGER);}
0[xX][0-9A-Fa-f]+  { yylval.str=strdup(yytext); print_token (yylval.str);
                     return (HEX_INTEGER); }
0[bB][0-1]+        { yylval.str=strdup(yytext); print_token (yylval.str);
                     return (BINARY_INTEGER); }
[-+]?([0-9]+([eE][-+]?[0-9]+)|([0-9]+\.[0-9]*([eE][-+]?[0-9]+)?))|(\.[0-9]+([eE][-+]?[0-9]+)?) { yylval.str=strdup(yytext);
                     print_token (yylval.str);
                     return(REAL);}

.                  { return(-1);}

%%
// This routine is the standard error trap if a lexical error occurs.

int yyerror(char* s)
{
  fprintf(stderr, "ERROR: %s at '%s' in statement %d.\n",
          s, yytext, statement_count);
  fprintf(messages, "ERROR: %s at '%s' in statement %d.\n",
          s, yytext, statement_count);
  return 0;
}
